# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-06-05 20:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0004_auto_20180604_1455'),
    ]

    operations = [
        migrations.AlterField(
            model_name='roboclelarshview',
            name='STATUS_FLG',
            field=models.CharField(max_length=4, null=True),
        ),
    ]
