# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-07-12 10:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0009_auto_20180621_0652'),
    ]

    operations = [
        migrations.CreateModel(
            name='RoboCxlView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('LOAN_ACCT_NUM', models.CharField(max_length=64)),
                ('CXL_SRL_NUM', models.CharField(max_length=48)),
                ('ENTITY_CRE_FLG', models.CharField(max_length=1)),
                ('DEL_FLG', models.CharField(max_length=1)),
                ('COMP_B2KID', models.CharField(max_length=260)),
                ('COMP_B2KID_TYPE', models.CharField(max_length=40)),
                ('SRL_NUM', models.CharField(max_length=16)),
                ('CHRG_TRAN_ID', models.CharField(max_length=36)),
                ('CHRG_PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('CHRG_TRAN_DATE', models.DateField(null=True)),
                ('PARENT_TRAN_ID', models.CharField(max_length=36)),
                ('PARENT_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('PARENT_TRAN_DATE', models.DateField(null=True)),
                ('EVENT_TYPE', models.CharField(max_length=20)),
                ('EVENT_ID', models.CharField(max_length=100)),
                ('SERVICE_SOL_ID', models.CharField(max_length=32)),
                ('SYSTEM_CALC_AMT', models.IntegerField(null=True)),
                ('ACTUAL_AMT_COLL', models.IntegerField(null=True)),
                ('COLL_CRNCY_CODE', models.CharField(max_length=12)),
                ('TRAN_PARTICULAR', models.CharField(max_length=200)),
                ('TRAN_RMKS', models.CharField(max_length=120)),
                ('REVERSAL_FLG', models.CharField(max_length=1)),
                ('START_DATE', models.DateField(null=True)),
                ('END_DATE', models.DateField(null=True)),
                ('TARGET_ACID', models.CharField(max_length=44)),
                ('CUST_ID', models.CharField(max_length=36)),
                ('CHRG_BORNE_BY_IND', models.CharField(max_length=1)),
                ('PART_TRAN_TYPE', models.CharField(max_length=1)),
                ('PTRAN_BUS_TYPE', models.CharField(max_length=20)),
                ('CHRG_ACID', models.CharField(max_length=44)),
                ('RATE_CODE1', models.CharField(max_length=20)),
                ('RATE_CODE2', models.CharField(max_length=20)),
                ('RATE_1', models.IntegerField(null=True)),
                ('RATE_2', models.IntegerField(null=True)),
                ('MODIFY_FLG', models.CharField(max_length=1)),
                ('LCHG_USER_ID', models.CharField(max_length=60)),
                ('LCHG_TIME', models.DateField(null=True)),
                ('RCRE_USER_ID', models.CharField(max_length=60)),
                ('RCRE_TIME', models.DateField(null=True)),
                ('CONSOL_DEBIT_PTRAN_FLG', models.CharField(max_length=1)),
                ('TS_CNT', models.IntegerField(null=True)),
                ('CHRG_RPT_CODE', models.CharField(max_length=20)),
                ('AMT_WITHOUT_DISCOUNT', models.IntegerField(null=True)),
                ('REC_NOT_EFFECTIVE', models.CharField(max_length=1)),
                ('CHANNEL_ID', models.CharField(max_length=20)),
                ('AMT_AFTER_FIRST_LVL_DISC', models.IntegerField(null=True)),
                ('RECORD_TYPE', models.CharField(max_length=1)),
                ('CONSOLIDATED_TRAN_SRL_NUM', models.IntegerField(null=True)),
                ('CHRG_GL_DATE', models.DateField(null=True)),
                ('PARENT_GL_DATE', models.DateField(null=True)),
                ('BANK_ID', models.CharField(max_length=32)),
                ('AMT_BEFORE_ADDTNL_DISC', models.IntegerField(null=True)),
                ('CHRG_DEBIT_PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('DEFAULT_CHRG_CALC_FLG', models.CharField(max_length=1)),
                ('REL_PRICING_REF_NUM', models.CharField(max_length=48)),
                ('GL_SEGMENT_STRING', models.CharField(max_length=200)),
                ('REF_CXL_SRL_NUM', models.CharField(max_length=48)),
                ('PARENT_CXL_SRL_NUM', models.CharField(max_length=48)),
                ('PART_CHRG_COLL_FLG', models.CharField(max_length=1)),
                ('PENDING_CHRG_AMT', models.IntegerField(null=True)),
                ('WAIVER_REASON_CODE', models.CharField(max_length=20)),
                ('WAIVER_REMARKS', models.CharField(max_length=120)),
                ('CHRG_REVERSAL_REASON_CODE', models.CharField(max_length=20)),
            ],
        ),
        migrations.CreateModel(
            name='RoboLdaView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('LOAN_ACCT_NUM', models.CharField(max_length=64)),
                ('ACID', models.CharField(max_length=44)),
                ('SHDL_NUM', models.CharField(max_length=8)),
                ('DMD_FLOW_ID', models.CharField(max_length=20)),
                ('DMD_DATE', models.DateField(null=True)),
                ('DMD_SRL_NUM', models.CharField(max_length=8)),
                ('SRL_NUM', models.CharField(max_length=12)),
                ('DEL_FLG', models.CharField(max_length=1)),
                ('ADJ_DATE', models.DateField(null=True)),
                ('ADJ_AMT', models.IntegerField(null=True)),
                ('LCHG_USER_ID', models.CharField(max_length=60)),
                ('LCHG_TIME', models.DateField(null=True)),
                ('RCRE_USER_ID', models.CharField(max_length=60)),
                ('RCRE_TIME', models.DateField(null=True)),
                ('OFLOW_ADJ_FLG', models.CharField(max_length=1)),
                ('TRAN_DATE', models.DateField(null=True)),
                ('TRAN_ID', models.CharField(max_length=36)),
                ('PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('TS_CNT', models.IntegerField(null=True)),
                ('BANK_ID', models.CharField(max_length=32)),
                ('WAIVER_SRL_NUM', models.CharField(max_length=16)),
                ('WAIVER_RECORD', models.CharField(max_length=1)),
            ],
        ),
        migrations.CreateModel(
            name='RoboLtdView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('LOAN_ACCT_NUM', models.CharField(max_length=64)),
                ('TRAN_DATE', models.DateField(null=True)),
                ('TRAN_ID', models.CharField(max_length=36)),
                ('PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('SHDL_NUM', models.CharField(max_length=8)),
                ('FLOW_ID', models.CharField(max_length=20)),
                ('ENTITY_CRE_FLG', models.CharField(max_length=1)),
                ('DEL_FLG', models.CharField(max_length=1)),
                ('ACID', models.CharField(max_length=44)),
                ('VALUE_DATE', models.DateField(null=True)),
                ('FLOW_AMT', models.IntegerField(null=True)),
                ('REVERSAL_FLG', models.CharField(max_length=1)),
                ('REVERSED_FLG', models.CharField(max_length=1)),
                ('DMD_OFFSET_FLG', models.CharField(max_length=1)),
                ('LCHG_USER_ID', models.CharField(max_length=60)),
                ('LCHG_TIME', models.DateField(null=True)),
                ('RCRE_USER_ID', models.CharField(max_length=60)),
                ('RCRE_TIME', models.DateField(null=True)),
                ('DMD_DATE', models.DateField(null=True)),
                ('LAST_TRAN_FLG', models.CharField(max_length=1)),
                ('LOAN_INT_ACID', models.CharField(max_length=44)),
                ('REVERSAL_REF_ID', models.CharField(max_length=200)),
                ('TYPE_OF_DMDS', models.CharField(max_length=1)),
                ('DMD_SRL_NUM', models.CharField(max_length=8)),
                ('ORIGIN_OF_TRAN', models.CharField(max_length=20)),
                ('OFLOW_ADJ_FLG', models.CharField(max_length=1)),
                ('TS_CNT', models.IntegerField(null=True)),
                ('PRINCIPAL_AMT', models.IntegerField(null=True)),
                ('INTEREST_AMT', models.IntegerField(null=True)),
                ('BANKCHARGE_AMT', models.IntegerField(null=True)),
                ('OTHERCHARGE_AMT', models.IntegerField(null=True)),
                ('DMD_SATISFY_FLG', models.CharField(max_length=1)),
                ('INT_BENEFIT_FLG', models.CharField(max_length=1)),
                ('ADVANCE_PAYMENT_FLG', models.CharField(max_length=1)),
                ('TRAN_VALUE_DATE', models.DateField(null=True)),
                ('INT_BENEFIT_DATE', models.DateField(null=True)),
                ('RAISE_DMD_FLG', models.CharField(max_length=1)),
                ('RAISE_DMD_FOR_AMT', models.IntegerField(null=True)),
                ('DEFERRED_INT_AMT', models.IntegerField(null=True)),
                ('DEMAND_IND', models.CharField(max_length=1)),
                ('PREPAYMENT_TYPE', models.CharField(max_length=1)),
                ('INT_COLL_ON_PREPAYMENT_FLG', models.CharField(max_length=1)),
                ('TRANSFER_TYPE', models.CharField(max_length=1)),
                ('TOP_DISB_FLG', models.CharField(max_length=1)),
                ('GL_DATE', models.DateField(null=True)),
                ('BANK_ID', models.CharField(max_length=32)),
                ('NORMAL_INTEREST_AMT', models.IntegerField(null=True)),
                ('PENAL_INTEREST_AMT', models.IntegerField(null=True)),
                ('OVERDUE_INTEREST_AMT', models.IntegerField(null=True)),
                ('INTEREST_OVERDUE_AMT', models.IntegerField(null=True)),
                ('HOLIDAY_INTEREST_AMT', models.IntegerField(null=True)),
            ],
        ),
    ]
