# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-12-04 07:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0016_misreportrequesttracking_requester'),
    ]

    operations = [
        migrations.AlterField(
            model_name='robolrsview',
            name='CURRENT_INT_REC_FLG',
            field=models.CharField(max_length=1, null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='GAM_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='GAM_LAST_CHANGE_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='LRS_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='NEXT_INT_DMD_DATE',
            field=models.DateField(null=True),
        ),
    ]
